@@ -21,6 +21,7 @@
 ******************************************************************************/
 
 #include "envy24control.h"
+#include "midi.h"
 #define _GNU_SOURCE
 #include <getopt.h>
 
@@ -189,7 +190,7 @@ static void create_mixer_frame(GtkWidget *box, int stream)
 	gtk_scale_set_digits(GTK_SCALE(vscale), 0);
 	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
 			   GTK_SIGNAL_FUNC(mixer_adjust),
-			  (gpointer)((stream << 16) + 0));
+			   (gpointer)(long)((stream << 16) + 0));
 
 	vbox1 = gtk_vbox_new(FALSE, 0);
 	gtk_widget_show(vbox1);
@@ -222,7 +223,7 @@ static void create_mixer_frame(GtkWidget *box, int stream)
 	gtk_scale_set_digits(GTK_SCALE(vscale), 0);
 	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
 			   GTK_SIGNAL_FUNC(mixer_adjust),
-			  (gpointer)((stream << 16) + 1));
+			   (gpointer)(long)((stream << 16) + 1));
 	
 	hbox = gtk_hbox_new(TRUE, 0);
 	gtk_widget_show(hbox);
@@ -255,7 +256,7 @@ static void create_mixer_frame(GtkWidget *box, int stream)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle), TRUE);
 	gtk_signal_connect(GTK_OBJECT(toggle), "toggled",
 			   GTK_SIGNAL_FUNC(mixer_toggled_mute),
-			  (gpointer)((stream << 16) + 0));
+			   (gpointer)(long)((stream << 16) + 0));
 
 	toggle = gtk_toggle_button_new_with_label("Mute");
 	mixer_mute_toggle[stream-1][1] = toggle;
@@ -264,7 +265,7 @@ static void create_mixer_frame(GtkWidget *box, int stream)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle), TRUE);
 	gtk_signal_connect(GTK_OBJECT(toggle), "toggled",
 			   GTK_SIGNAL_FUNC(mixer_toggled_mute),
-			  (gpointer)((stream << 16) + 1));
+			   (gpointer)(long)((stream << 16) + 1));
 }
 
 static void create_mixer(GtkWidget *main, GtkWidget *notebook, int page)
@@ -453,7 +454,7 @@ static void create_router_frame(GtkWidget *box, int stream, int pos)
 	gtk_box_pack_start(GTK_BOX(vbox), radiobutton, FALSE, FALSE, 0);
 	gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
 			  (GtkSignalFunc)patchbay_toggled, 
-			  (gpointer)((stream << 16) + 0));
+			   (gpointer)(long)((stream << 16) + 0));
 
 
 	hseparator = gtk_hseparator_new();
@@ -463,7 +464,10 @@ static void create_router_frame(GtkWidget *box, int stream, int pos)
 	label = gtk_label_new("");
 	gtk_widget_show(label);
 
-	if(stream <= MAX_OUTPUT_CHANNELS + MAX_SPDIF_CHANNELS) {
+	/* the digital mixer can only be routed to HW1/2 or SPDIF1/2 */
+	if( (stream <= 2) /* hw1/2 */ ||
+	    ((stream > MAX_OUTPUT_CHANNELS) && (stream <= MAX_OUTPUT_CHANNELS + 2)) /* spdif1/2 */
+	    ) {
 		radiobutton = gtk_radio_button_new_with_label(group, stream & 1 ? "Digital Mix L" : "Digital Mix R");
 		router_radio[stream-1][1] = radiobutton;
 		group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
@@ -472,7 +476,7 @@ static void create_router_frame(GtkWidget *box, int stream, int pos)
 				    radiobutton, FALSE, FALSE, 0);
 		gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
 				  (GtkSignalFunc)patchbay_toggled, 
-				  (gpointer)((stream << 16) + 1));
+				   (gpointer)(long)((stream << 16) + 1));
 	}
 	else {
 	  label = gtk_label_new("");
@@ -495,7 +499,7 @@ static void create_router_frame(GtkWidget *box, int stream, int pos)
 				    radiobutton, FALSE, FALSE, 0);
 		gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
 				  (GtkSignalFunc)patchbay_toggled, 
-				  (gpointer)((stream << 16) + 2 + idx));
+				   (gpointer)(long)((stream << 16) + 2 + idx));
 	}
 }
 
@@ -1544,7 +1548,7 @@ static void create_analog_volume(GtkWidget *main, GtkWidget *notebook, int page)
 		gtk_scale_set_digits(GTK_SCALE(vscale), 0);
 		gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
 				   GTK_SIGNAL_FUNC(dac_volume_adjust), 
-				  (gpointer)(i));
+				   (gpointer)(long)(i));
 
 	        label = gtk_label_new("000");
 	        av_dac_volume_label[i] =(GtkLabel *)label;
@@ -1562,7 +1566,7 @@ static void create_analog_volume(GtkWidget *main, GtkWidget *notebook, int page)
 			gtk_widget_show(radiobutton);
 			gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
 					  (GtkSignalFunc)dac_sense_toggled, 
-					  (gpointer)((i << 8) + j));
+					   (gpointer)(long)((i << 8) + j));
 			gtk_box_pack_start(GTK_BOX(vbox), 
 					    radiobutton, FALSE, TRUE, 0);
 			group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
@@ -1601,7 +1605,7 @@ static void create_analog_volume(GtkWidget *main, GtkWidget *notebook, int page)
 		gtk_scale_set_digits(GTK_SCALE(vscale), 0);
 		gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
 				   GTK_SIGNAL_FUNC(adc_volume_adjust), 
-				  (gpointer)(i));
+				   (gpointer)(long)(i));
 
 	        label = gtk_label_new("000");
 	        av_adc_volume_label[i] =(GtkLabel *)label;
@@ -1618,7 +1622,7 @@ static void create_analog_volume(GtkWidget *main, GtkWidget *notebook, int page)
 			gtk_widget_show(radiobutton);
 			gtk_signal_connect(GTK_OBJECT(radiobutton), "toggled",
 					  (GtkSignalFunc)adc_sense_toggled, 
-					  (gpointer)((i << 8) + j));
+					   (gpointer)(long)((i << 8) + j));
 			gtk_box_pack_start(GTK_BOX(vbox), 
 					    radiobutton, FALSE, TRUE, 0);
 			group = gtk_radio_button_group(GTK_RADIO_BUTTON(radiobutton));
@@ -1657,7 +1661,7 @@ static void create_analog_volume(GtkWidget *main, GtkWidget *notebook, int page)
 		gtk_scale_set_digits(GTK_SCALE(vscale), 0);
 		gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
 				   GTK_SIGNAL_FUNC(ipga_volume_adjust), 
-				  (gpointer)(i));
+				   (gpointer)(long)(i));
 
 	        label = gtk_label_new("000");
 	        av_ipga_volume_label[i] = (GtkLabel *)label;
@@ -1926,7 +1930,7 @@ static void create_profiles(GtkWidget *main, GtkWidget *notebook, int page)
 
 static void usage(void)
 {
-	fprintf(stderr, "usage: envy24control [-c card#] [-D control-name] [-o num-outputs] [-i num-inputs] [-p num-pcm-outputs] [-s num-spdif-in/outs] [-v] [-f profiles-file] [profile name|profile id]\n");
+	fprintf(stderr, "usage: envy24control [-c card#] [-D control-name] [-o num-outputs] [-i num-inputs] [-p num-pcm-outputs] [-s num-spdif-in/outs] [-v] [-f profiles-file] [profile name|profile id] [-m channel-num]\n");
 	fprintf(stderr, "\t-c, --card\tAlsa card number to control\n");
 	fprintf(stderr, "\t-D, --device\tcontrol-name\n");
 	fprintf(stderr, "\t-o, --outputs\tLimit number of analog line outputs to display\n");
@@ -1935,6 +1939,7 @@ static void usage(void)
 	fprintf(stderr, "\t-s, --spdif\tLimit number of spdif inputs/outputs to display\n");
 	fprintf(stderr, "\t-v, --view_spdif_playback\tshows the spdif playback channels in the mixer\n");
 	fprintf(stderr, "\t-f, --profiles_file\tuse file as profiles file\n");
+	fprintf(stderr, "\t-m, --midichannel\tmidi channel number for controller control\n");
 }
 
 int main(int argc, char **argv)
@@ -1946,18 +1951,21 @@ int main(int argc, char **argv)
 	snd_ctl_elem_value_t *val;
 	int npfds;
 	struct pollfd *pfds;
+	int midi_fd = -1, midi_channel = -1;
 	int page;
 	int input_channels_set = 0;
 	int output_channels_set = 0;
 	static struct option long_options[] = {
 		{"device", 1, 0, 'D'},
 		{"card", 1, 0, 'c'},
+		{"profiles_file", 1, 0, 'f'},
 		{"inputs", 1, 0, 'i'},
+		{"midichannel", 1, 0, 'm'},
 		{"outputs", 1, 0, 'o'},
 		{"pcm_outputs", 1, 0, 'p'},
 		{"spdif", 1, 0, 's'},
-		{"profiles_file", 1, 0, 'f'},
 		{"view_spdif_playback", 0, 0, 'v'},
+		{ NULL }
 	};
 
 
@@ -1967,7 +1975,7 @@ int main(int argc, char **argv)
 	/* Go through gtk initialization */
         gtk_init(&argc, &argv);
 
-	name = "hw:0";
+	name = NULL; /* probe */
 	card_number = 0;
 	input_channels = MAX_INPUT_CHANNELS;
 	output_channels = MAX_OUTPUT_CHANNELS;
@@ -1976,8 +1984,16 @@ int main(int argc, char **argv)
 	view_spdif_playback = 0;
 	profiles_file_name = DEFAULT_PROFILERC;
 	default_profile = NULL;
-	while ((c = getopt_long(argc, argv, "D:c:i:o:p:s:f:v", long_options, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "D:c:f:i:m:o:p:s:v", long_options, NULL)) != -1) {
 		switch (c) {
+		case 'D':
+			name = optarg;
+			card_number = atoi(strchr(name, ':') + sizeof(char));
+			if (card_number < 0 || card_number >= MAX_CARD_NUMBERS) {
+				fprintf(stderr, "envy24control: invalid card number %d\n", card_number);
+				exit(1);
+			}
+			break;
 		case 'c':
 			i = atoi(optarg);
 			if (i < 0 || i >= MAX_CARD_NUMBERS) {
@@ -1988,13 +2004,8 @@ int main(int argc, char **argv)
 			sprintf(tmpname, "hw:%d", i);
 			name = tmpname;
 			break;
-		case 'D':
-			name = optarg;
-			card_number = atoi(strchr(name, ':') + sizeof(char));
-			if (card_number < 0 || card_number >= MAX_CARD_NUMBERS) {
-				fprintf(stderr, "envy24control: invalid card number %d\n", card_number);
-				exit(1);
-			}
+		case 'f':
+			profiles_file_name = optarg;
 			break;
 		case 'i':
 			input_channels = atoi(optarg);
@@ -2004,6 +2015,14 @@ int main(int argc, char **argv)
 			}
 			input_channels_set = 1;
 			break;
+		case 'm':
+			midi_channel = atoi(optarg);
+			if (midi_channel < 1 || midi_channel > 16) {
+				fprintf(stderr, "envy24control: invalid midi channel number %i\n", midi_channel);
+				exit(1);
+			}
+			--midi_channel;
+			break;
 		case 'o':
 			output_channels = atoi(optarg);
 			if (output_channels < 0 || output_channels > MAX_OUTPUT_CHANNELS) {
@@ -2026,9 +2045,6 @@ int main(int argc, char **argv)
 				exit(1);
 			}
 			break;
-		case 'f':
-			profiles_file_name = optarg;
-			break;
 		case 'v':
 			view_spdif_playback = 1;
 			break;
@@ -2042,17 +2058,40 @@ int main(int argc, char **argv)
 		default_profile = argv[optind];
 	}
 
-	if ((err = snd_ctl_open(&ctl, name, 0)) < 0) {
-		fprintf(stderr, "snd_ctl_open: %s\n", snd_strerror(err));
-		exit(EXIT_FAILURE);
-	}
-	if ((err = snd_ctl_card_info(ctl, hw_info)) < 0) {
-		fprintf(stderr, "snd_ctl_card_info: %s\n", snd_strerror(err));
-		exit(EXIT_FAILURE);
-	}
-	if (strcmp(snd_ctl_card_info_get_driver(hw_info), "ICE1712")) {
-		fprintf(stderr, "invalid card type (driver is %s)\n", snd_ctl_card_info_get_driver(hw_info));
-		exit(EXIT_FAILURE);
+	if (! name) {
+		/* probe cards */
+		static char cardname[8];
+		/* FIXME: hardcoded max number of cards */
+		for (card_number = 0; card_number < 8; card_number++) {
+			sprintf(cardname, "hw:%d", card_number);
+			if (snd_ctl_open(&ctl, cardname, 0) < 0)
+				continue;
+			if (snd_ctl_card_info(ctl, hw_info) < 0 ||
+			    strcmp(snd_ctl_card_info_get_driver(hw_info), "ICE1712")) {
+				snd_ctl_close(ctl);
+				continue;
+			}
+			/* found */
+			name = cardname;
+			break;
+		}
+		if (! name) {
+			fprintf(stderr, "No ICE1712 cards found\n");
+			exit(EXIT_FAILURE);
+		}
+	} else {
+		if ((err = snd_ctl_open(&ctl, name, 0)) < 0) {
+			fprintf(stderr, "snd_ctl_open: %s\n", snd_strerror(err));
+			exit(EXIT_FAILURE);
+		}
+		if ((err = snd_ctl_card_info(ctl, hw_info)) < 0) {
+			fprintf(stderr, "snd_ctl_card_info: %s\n", snd_strerror(err));
+			exit(EXIT_FAILURE);
+		}
+		if (strcmp(snd_ctl_card_info_get_driver(hw_info), "ICE1712")) {
+			fprintf(stderr, "invalid card type (driver is %s)\n", snd_ctl_card_info_get_driver(hw_info));
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	snd_ctl_elem_value_set_interface(val, SND_CTL_ELEM_IFACE_CARD);
@@ -2082,6 +2121,8 @@ int main(int argc, char **argv)
 	patchbay_init();
 	hardware_init();
 	analog_volume_init();
+	if (midi_channel >= 0)
+		midi_fd = midi_init(argv[0], midi_channel);
 
 	fprintf(stderr, "using\t --- input_channels: %i\n\t --- output_channels: %i\n\t --- pcm_output_channels: %i\n\t --- spdif in/out channels: %i\n", \
 		input_channels, output_channels, pcm_output_channels, spdif_channels);
@@ -2122,6 +2163,9 @@ int main(int argc, char **argv)
 				      ctl);
 		snd_ctl_subscribe_events(ctl, 1);
 	}
+	if (midi_fd >= 0) {
+		gdk_input_add(midi_fd, GDK_INPUT_READ, midi_process, NULL);
+	}
 	gtk_timeout_add(40, level_meters_timeout_callback, NULL);
 	gtk_timeout_add(100, master_clock_status_timeout_callback, NULL);
 	gtk_timeout_add(100, internal_clock_status_timeout_callback, NULL);
@@ -2140,6 +2184,7 @@ int main(int argc, char **argv)
 	gtk_main();
 
 	snd_ctl_close(ctl);
+	midi_close();
 
 	return EXIT_SUCCESS;
 }
