@@ -7,6 +7,25 @@
 #include <gtk/gtk.h>
 #include <alsa/asoundlib.h>
 
+/* Profiles */
+#ifdef PACKAGE
+#define PROGRAM_NAME PACKAGE
+#else
+#define PROGRAM_NAME "envy24control"
+#endif
+#define MAX_PROFILES 8
+#define MAX_PROFILE_NAME_LENGTH 20
+#define DEFAULT_PROFILERC "~/envy24control/profiles.conf"
+#define SYS_PROFILERC "/etc/envy24control/profiles.conf"
+#ifndef MKDIR
+#define MKDIR "/bin/mkdir"
+#endif
+#ifndef ALSACTL
+#define ALSACTL "/usr/sbin/alsactl"
+#endif
+
+#include "profiles.h"
+
 /* MidiMan */
 #define ICE1712_SUBDEVICE_DELTA1010	0x121430d6
 #define ICE1712_SUBDEVICE_DELTADIO2496	0x121431d6
@@ -170,6 +189,8 @@ void patchbay_postinit(void);
 void master_clock_update(void);
 gint master_clock_status_timeout_callback(gpointer data);
 gint internal_clock_status_timeout_callback(gpointer data);
+gint rate_locking_status_timeout_callback(gpointer data);
+gint rate_reset_status_timeout_callback(gpointer data);
 void internal_clock_toggled(GtkWidget *togglebutton, gpointer data);
 void rate_locking_update(void);
 void rate_locking_toggled(GtkWidget *togglebutton, gpointer data);
